"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = exports.expressApp = void 0;
require("reflect-metadata");
const express_1 = __importStar(require("express"));
const body_parser_1 = __importDefault(require("body-parser"));
const dotenv_1 = __importDefault(require("dotenv"));
const path = __importStar(require("path"));
const commons_1 = require("../commons");
const node_cron_1 = require("node-cron");
const fs_1 = require("fs");
const ws_1 = require("ws");
const metadata_1 = require("../global/metadata");
const mongodb_1 = require("../connections/mongodb");
const migrations_1 = require("../database/migrations");
const metadata_2 = require("../metadata");
const decorators_1 = require("../decorators");
const helpers_1 = require("../helpers");
const authorization_1 = require("../authorization");
const injector_1 = require("../di/injector");
const services_1 = require("../services");
const configuration_1 = require("../configs/configuration");
const consts_1 = require("../authorization/consts");
const di_1 = require("../di");
const injectResponse_1 = require("../responseBuilder/injectResponse");
const rootPath = path.dirname(require.main.filename);
dotenv_1.default.config();
class HttpException extends Error {
    constructor(statusCode, message) {
        super(message);
        this.statusCode = statusCode;
        this.message = message;
    }
}
exports.expressApp = express_1.default;
class Application {
    constructor(config, isTest) {
        this.config = config;
        this.isTest = isTest;
        this.info = [];
        this.errorHandler = (error, request, response, next) => {
            const status = error.statusCode || 500;
            const message = error.message || "Something went wrong";
            const name = error.name;
            response._error = error;
            response.status(status).broadcast({
                status: false,
                message,
                name
            });
        };
        injector_1.injector.set(configuration_1.Configuration, config);
        this.app = (0, express_1.default)();
        this.app.use((req, res, next) => (0, injectResponse_1.injectorResponse)(req, res, next, {
            _isTest: isTest,
            _logger: new commons_1.WinstonService(config.IS_LOG_FILE)
        }));
        this.inital();
    }
    inital() {
        const config = injector_1.injector.get(configuration_1.Configuration);
        if (config.REDIS_URL) {
            injector_1.injector.set(services_1.RedisServer, new services_1.RedisServer(config.REDIS_URL));
            const redisClient = injector_1.injector.get(services_1.RedisServer).client;
            redisClient.connect();
            redisClient.on("connect", () => {
                if (!this.isTest) {
                    console.log("****************************");
                    console.log(`*    ${(0, commons_1.cinfo)("Start Redis Server")}`);
                    console.log(`*    ${(0, commons_1.cinfo)("REDIS_URL: " + config.REDIS_URL)}`);
                    console.log(`*    ${(0, commons_1.csuccess)("Status: Connected")}`);
                    console.log("****************************\n");
                }
            });
            redisClient.on("error", (error) => {
                if (!this.isTest) {
                    console.log("****************************");
                    console.log(`*    ${(0, commons_1.cinfo)("Start Redis Server")}`);
                    console.log(`*    ${(0, commons_1.cinfo)("REDIS_URL: " + config.REDIS_URL)}`);
                    console.log(`*    ${(0, commons_1.cerror)("Status: Failed")}`);
                    console.log(`*    ${(0, commons_1.cerror)("Error: ")}`, (0, commons_1.cerror)(error));
                    console.log("****************************\n");
                }
                process.exit(2);
            });
        }
        if (config.PORT_SOCKET) {
            injector_1.injector.set(ws_1.WebSocketServer, new ws_1.WebSocketServer({ port: config.PORT_SOCKET }));
            if (!this.isTest) {
                console.log("****************************");
                console.log(`*    ${(0, commons_1.cinfo)("Start Socket Server")}`);
                console.log(`*    ${(0, commons_1.cinfo)("PORT_SOCKET: " + config.PORT_SOCKET)}`);
                console.log("****************************\n");
            }
        }
    }
    generateRole(configRole) {
        return __awaiter(this, void 0, void 0, function* () {
            const roleNames = configRole.split(",");
            if (!roleNames.includes(consts_1.RoleAdmin)) {
                roleNames.push(consts_1.RoleAdmin);
            }
            yield Promise.all(roleNames.map((roleName) => __awaiter(this, void 0, void 0, function* () {
                let role = yield (0, helpers_1.getQueryModel)(authorization_1.Role).findOne({ name: { $eq: roleName } });
                if (!role) {
                    role = yield (0, helpers_1.getQueryModel)(authorization_1.Role).createOne({ name: roleName });
                }
                console.log(role);
            })));
        });
    }
    generatePermisson(roleName, api) {
        return __awaiter(this, void 0, void 0, function* () {
            const role = yield (0, helpers_1.getQueryModel)(authorization_1.Role).findOne({ name: roleName });
            if (!role) {
                console.log("No roles found");
                process.exit(1);
            }
            let permission = yield (0, helpers_1.getQueryModel)(authorization_1.Permission).findOne({ method: { $eq: api.method }, api: { $eq: api.path } });
            if (!permission) {
                permission = yield (0, helpers_1.getQueryModel)(authorization_1.Permission).createOne({
                    method: api.method,
                    api: api.path,
                    name: ""
                });
            }
            let data = yield (0, helpers_1.getQueryModel)(authorization_1.RoleHasPermission).findOne({
                roleId: { $eq: role._id },
                permissionId: { $eq: permission._id }
            });
            if (!data) {
                data = yield (0, helpers_1.getQueryModel)(authorization_1.RoleHasPermission).createOne({
                    roleId: role._id,
                    permissionId: permission._id
                });
            }
            console.log(data);
        });
    }
    publishDocApi() {
        const pathDocJson = rootPath + "/api-doc.json";
        if ((0, fs_1.existsSync)(pathDocJson)) {
            const swaggerUi = require("swagger-ui-express");
            const swaggerDocument = require(pathDocJson);
            this.app.use("/api-docs", swaggerUi.serve, swaggerUi.setup(swaggerDocument));
            this.info.push({
                method: "GET",
                api: "/api-docs",
                handler: "swagger",
                default: true,
                isPublic: true
            });
        }
    }
    /**
     *
     * @param req Request of express
     * @param res Response of express
     * @param next Next function of express
     * @param paramConfigs Parameters
     * @param handlerName The name of the handler
     * @param controller The controller
     */
    apiHandler(req, res, next, paramConfigs, handlerName, controller) {
        const args = (0, decorators_1.extractParameters)(req, res, next, paramConfigs);
        try {
            controller.instance[String(handlerName)]
                .apply(controller.instance, args)
                .then((data) => {
                res.broadcast(data);
                return;
            })
                .catch((e) => {
                this.errorHandler(e, req, res, next);
                return;
            });
        }
        catch (error) {
            this.errorHandler(error, req, res, next);
            return;
        }
    }
    onExit() {
        process.on('beforeExit', (code) => __awaiter(this, void 0, void 0, function* () {
            console.log('Process beforeExit event with code: ', code);
            yield this.handleOnExist();
            process.exit(0);
        }));
        process.on('exit', (code) => __awaiter(this, void 0, void 0, function* () {
            console.log('Process exit event with code: ', code);
            yield this.handleOnExist();
            process.exit(0);
        }));
        process.on('SIGTERM', (signal) => __awaiter(this, void 0, void 0, function* () {
            console.log(`Process ${process.pid} received a SIGTERM signal`, signal);
            yield this.handleOnExist();
            process.exit(0);
        }));
        process.on('SIGINT', (signal) => __awaiter(this, void 0, void 0, function* () {
            console.log(`Process ${process.pid} has been interrupted`, signal);
            yield this.handleOnExist();
            process.exit(0);
        }));
    }
    runRouter(callback) {
        this.app.use('/uploads', express_1.default.static('uploads'));
        this.app.use(body_parser_1.default.json());
        (0, metadata_1.getMetadataController)().forEach((controller) => {
            const routers = Reflect.getMetadata(metadata_2.MetadataKeys.ROUTERS, controller.constructorFn) || [];
            const basePath = Reflect.getMetadata(metadata_2.MetadataKeys.BASE_PATH, controller.constructorFn);
            const exRouter = (0, express_1.Router)();
            routers.forEach(({ method, path, handlerName, isDefault, isPublic, isAbsolute }) => {
                if (!isDefault && !isAbsolute) {
                    path = basePath + path;
                }
                const paramConfigs = Reflect.getMetadata(metadata_2.MetadataKeys.ROUTE_ARGS_METADATA, controller.constructorFn, handlerName);
                const middlewares = Reflect.getMetadata(metadata_2.MetadataKeys.MIDDLEWARE, controller.constructorFn, handlerName) || [];
                const handler = [];
                if (paramConfigs && paramConfigs.find((e) => e.type == decorators_1.RouteParamtypes.UPLOAD)) {
                    handler.push(helpers_1.upload.single("file"));
                }
                if (!isPublic) {
                    handler.push((req, res, next) => (0, authorization_1.middlewareAccessApi)(req, res, next, { method, path }, callback));
                }
                middlewares.forEach((middleware) => {
                    handler.push(middleware);
                });
                handler.push((req, res, next) => this.apiHandler(req, res, next, paramConfigs, handlerName, controller));
                exRouter[method](path, ...handler);
                this.info.push({
                    method: `${method.toLocaleUpperCase()}`,
                    api: `${path}`,
                    handler: `${controller.constructorFn.name}.${String(handlerName)}`,
                    default: !!isDefault,
                    isPublic: !!isPublic
                });
            });
            this.app.use(exRouter);
        });
    }
    runScheduler() {
        (0, metadata_1.getMetadataScheduler)().forEach(scheduler => {
            const tasks = Reflect.getMetadata(metadata_2.MetadataKeys.SCHEDULER, scheduler) || [];
            tasks.forEach((task) => {
                try {
                    const instanceScheduler = (0, di_1.getServiceProvider)().getInfo(scheduler).singletonInstance;
                    (0, node_cron_1.schedule)(task.expression, () => {
                        instanceScheduler[task.handlerName]
                            .apply(instanceScheduler)
                            .then(() => {
                            const taskSuccess = (0, commons_1.csuccess)(`[${scheduler.name}-${task.handlerName}]`);
                            console.log(`${taskSuccess}[${Date.now()}] run successfully`);
                        })
                            .catch((e) => {
                            const message = e.message || "Something went wrong";
                            const taskError = (0, commons_1.cerror)(`[${scheduler.name}-${task.handlerName}]`);
                            console.log(`${taskError}[${Date.now()}] error: ${message}`);
                        });
                    }, task.option);
                }
                catch (error) {
                    const message = error.message || "Something went wrong";
                    const taskError = (0, commons_1.cerror)(`[${scheduler.name}-${task.handlerName}]`);
                    console.log(`${taskError}[${Date.now()}] error: ${message}`);
                }
            });
        });
    }
    handleOnExist() {
        return __awaiter(this, void 0, void 0, function* () {
            // clear socket in redis
            const redisClient = injector_1.injector.get(services_1.RedisServer);
            if (redisClient) {
                yield redisClient.client.del("_SOCKET");
            }
        });
    }
    run(config) {
        return __awaiter(this, void 0, void 0, function* () {
            yield (0, mongodb_1.connectMongoDb)(this.config, this.isTest);
            const configRoles = process.env.npm_config_roles;
            const configRole = process.env.npm_config_role;
            const configPermisson = process.env.npm_config_permission;
            const configDB = process.env.npm_config_db;
            if (configRoles) {
                yield this.generateRole(configRoles);
                process.exit(1);
            }
            if (configPermisson && configRole) {
                const [method, path] = configPermisson.split("|");
                yield this.generatePermisson(configRole, { method, path });
                process.exit(1);
            }
            else if (configDB) {
                yield (0, migrations_1.compareDBMetadata)();
                process.exit(1);
            }
            else {
                this.runRouter(config.middlwareAuthExtend || null);
                this.publishDocApi();
                if (config.logDocApi && !this.isTest) {
                    console.table(this.info);
                }
                const port = this.config.PORT || 8000;
                this.app.listen(port, () => {
                    if (!this.isTest) {
                        console.log(`[server]: Server is running at http://localhost:${port}`);
                    }
                });
                const configApp = injector_1.injector.get(configuration_1.Configuration);
                if (configApp.PORT_SOCKET) {
                    new services_1.ApplicationWebSocket(injector_1.injector.get(services_1.AbstractSocketConnectionHandler), injector_1.injector.get(ws_1.WebSocketServer), injector_1.injector.get(services_1.RedisServer)).run();
                }
                this.runScheduler();
            }
            this.onExit();
        });
    }
}
exports.Application = Application;
