"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateParamDefault = void 0;
const index_1 = require("../../../../index");
const getDocDTO_1 = require("./getDocDTO");
const getDocAdditionQueryModel = (classModel) => {
    const data = (0, index_1.getMetadataTable)()
        .getColumn(classModel)
        .reduce((target, item) => {
        var _a;
        if (item.isEnum) {
            const enumValues = JSON.stringify(Object.values((_a = item.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType));
            // Available values
            target += `<li><strong>${item.name}</strong>: Available values ${enumValues}</li>`;
        }
        return target;
    }, "");
    return `<ul>${data}</ul>`;
};
const getDocQueryModel = (classModel) => {
    const data = (0, index_1.getMetadataTable)()
        .getColumn(classModel)
        .reduce((target, item) => {
        switch (item.type) {
            case Date:
                target[item.name] = "";
                break;
            case String:
                target[item.name] = "";
                break;
            case Number:
                target[item.name] = {
                    min: 0,
                    max: 1,
                };
                break;
            case index_1.ObjectId:
                target[item.name] = [];
                break;
            case Boolean:
                target[item.name] = true;
                break;
            default:
                break;
        }
        return target;
    }, {});
    return JSON.stringify(data);
};
const handleParamQuery = (classModel) => {
    return [
        {
            name: "queries",
            description: "filter condition",
            in: "query",
        },
        {
            name: "paginate",
            description: `Paginate of list ${classModel.name}, default limit is 20, default page is 1`,
            in: "query",
        },
    ];
};
const handleParamBody = (classModel, typeFunction) => {
    const data = (0, index_1.getMetadataTable)()
        .getColumn(classModel)
        .reduce((target, field) => {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        if (!["_id", "updatedAt", "createdAt"].includes(field.name)) {
            let type = [String, Number, Boolean, Array].includes(field.type)
                ? field.type.name.toLocaleLowerCase()
                : [index_1.ObjectId, Date].includes(field.type)
                    ? "string"
                    : "object";
            if (type === "object" && field.isEnum) {
                type = "string";
            }
            target.properties[field.name] = {
                type,
                description: field.description,
                enum: field.isEnum ? Object.values((_a = field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType) : undefined
            };
            // if type is object then assign properties
            if (type == "object") {
                const docDto = (0, getDocDTO_1.getDocDTO)(field.type);
                target.properties[field.name].properties = docDto.result;
                target.properties[field.name].required = docDto.required;
            }
            // if type is array then assign items
            if (type == "array") {
                if (((_b = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _b === void 0 ? void 0 : _b.elementDataType) &&
                    [index_1.DataType.BOOLEAN, index_1.DataType.NUMBER, index_1.DataType.STRING].includes((_c = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _c === void 0 ? void 0 : _c.elementDataType)) {
                    target.properties[field.name].items = {
                        type: field.isEnum ? "string" : ((_d = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _d === void 0 ? void 0 : _d.elementDataType).toLocaleLowerCase(),
                        enum: field.isEnum ? Object.values((_e = field.vectorProps) === null || _e === void 0 ? void 0 : _e.elementDataType) : undefined
                    };
                }
                else {
                    const docDTO = (0, getDocDTO_1.getDocDTO)((_f = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _f === void 0 ? void 0 : _f.elementDataType);
                    target.properties[field.name].items = {
                        type: ((_g = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _g === void 0 ? void 0 : _g.elementDataType) && !field.isEnum ? "object" : "string",
                        properties: docDTO.result,
                        required: docDTO.required,
                        enum: field.isEnum ? Object.values((_h = field.vectorProps) === null || _h === void 0 ? void 0 : _h.elementDataType) : undefined
                    };
                }
            }
            if (typeFunction == "create" && field.isRequire) {
                target.required.push(field.name);
            }
        }
        return target;
    }, { properties: {}, required: [] });
    return {
        content: {
            "application/json": {
                schema: Object.assign({ type: "object" }, data),
            },
        },
    };
};
const handleParamResponse = (classModel, handlerName) => {
    if (handlerName != "get") {
        return {
            "200": {
                description: "Successfully",
            },
        };
    }
    const data = (0, index_1.getMetadataTable)()
        .getColumn(classModel)
        .reduce((target, field) => {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        let type = [String, Number, Boolean, Array].includes(field.type)
            ? field.type.name.toLocaleLowerCase()
            : [index_1.ObjectId, Date].includes(field.type)
                ? "string"
                : "object";
        if (type === "object" && field.isEnum) {
            type = "string";
        }
        target.properties[field.name] = {
            type,
            description: field.description,
            enum: field.isEnum ? Object.values((_a = field.vectorProps) === null || _a === void 0 ? void 0 : _a.elementDataType) : undefined
        };
        if (field.isRequire) {
            target.required.push(field.name);
        }
        // if type is object then assign properties
        if (type == "object") {
            const docDto = (0, getDocDTO_1.getDocDTO)(field.type);
            target.properties[field.name].properties = docDto.result;
            target.properties[field.name].required = docDto.required;
        }
        // if type is array then assign items
        if (type == "array") {
            if (((_b = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _b === void 0 ? void 0 : _b.elementDataType) &&
                [index_1.DataType.BOOLEAN, index_1.DataType.NUMBER, index_1.DataType.STRING].includes((_c = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _c === void 0 ? void 0 : _c.elementDataType)) {
                target.properties[field.name].items = {
                    type: field.isEnum ? "string" : ((_d = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _d === void 0 ? void 0 : _d.elementDataType).toLocaleLowerCase(),
                    enum: field.isEnum ? Object.values((_e = field.vectorProps) === null || _e === void 0 ? void 0 : _e.elementDataType) : undefined
                };
            }
            else {
                const docDTO = (0, getDocDTO_1.getDocDTO)((_f = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _f === void 0 ? void 0 : _f.elementDataType);
                target.properties[field.name].items = {
                    type: ((_g = field === null || field === void 0 ? void 0 : field.vectorProps) === null || _g === void 0 ? void 0 : _g.elementDataType) && !field.isEnum ? "object" : "string",
                    properties: docDTO.result,
                    required: docDTO.required,
                    enum: field.isEnum ? Object.values((_h = field.vectorProps) === null || _h === void 0 ? void 0 : _h.elementDataType) : undefined
                };
            }
        }
        return target;
    }, { properties: {}, required: [] });
    return {
        "200": {
            description: "Successfully",
            content: {
                "application/json": {
                    schema: {
                        type: "object",
                        properties: {
                            data: {
                                type: "array",
                                items: Object.assign({ type: "object" }, data),
                            },
                            totalCount: {
                                type: "number",
                                description: `Total record`,
                            },
                        },
                    },
                },
            },
        },
    };
};
const getDescriptionApi = (classModel, handlerName) => {
    const exampleQuery = getDocQueryModel(classModel);
    const docAdditionQuery = getDocAdditionQueryModel(classModel);
    const examplePaginate = JSON.stringify({ page: 1, limit: 20 });
    switch (handlerName) {
        case "get":
            return `Get list ${classModel.name}
            <br/> example: 
            <ul> 
                <li>queries: ${exampleQuery}</li>
                <li> paginate: ${examplePaginate} </li>
            </ul>
            <strong><i>*Note:</i></strong>
            ${docAdditionQuery}`;
        case "create":
            return `Create ${classModel.name}`;
        case "update":
            return `Update ${classModel.name}, example queries: ${exampleQuery}, paginate: ${examplePaginate}`;
        case "delete":
            return `Delete ${classModel.name}, example queries: ${exampleQuery}, paginate: ${examplePaginate}`;
        default:
            break;
    }
};
const generateParamDefault = (controller, handlerName) => {
    const paramTypes = Reflect.getMetadata(index_1.MetadataKeys.PARAM_TYPE, controller.constructorFn) || [];
    const requestDoc = {
        requestBody: ["create", "update"].includes(handlerName) ? handleParamBody(paramTypes[0], handlerName) : {},
        parameters: ["get", "update", "delete"].includes(handlerName) ? handleParamQuery(paramTypes[0]) : [],
        requestResponse: handleParamResponse(paramTypes[0], handlerName),
    };
    return {
        description: getDescriptionApi(paramTypes[0], handlerName),
        requestBody: requestDoc.requestBody,
        parameters: requestDoc.parameters,
        requestResponse: requestDoc.requestResponse,
    };
};
exports.generateParamDefault = generateParamDefault;
