"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiDoc = void 0;
require("reflect-metadata");
const index_1 = require("../../../../index");
const fs = __importStar(require("fs"));
const child_process_1 = require("child_process");
const generateParamCustomize_1 = require("./generateParamCustomize");
const generateParamDefault_1 = require("./generateParamDefault");
class ApiDoc {
    constructor(configDoc) {
        this.configDoc = configDoc;
    }
    run() {
        const info = [];
        const content = {
            openapi: "3.0.0",
            info: {
                title: this.configDoc.title,
                description: this.configDoc.description,
                version: this.configDoc.version,
            },
            externalDocs: this.configDoc.urlDownload ? {
                description: "download json",
                url: this.configDoc.urlDownload
            } : {},
            paths: {},
            components: {
                securitySchemes: {
                    bearerAuth: {
                        type: "http",
                        scheme: "bearer",
                        bearerFormat: "JWT",
                        description: "Enter the token, e.g. `abcde12345`"
                    }
                }
            },
        };
        (0, index_1.getMetadataController)().forEach((controller) => {
            const routers = Reflect.getMetadata(index_1.MetadataKeys.ROUTERS, controller.constructorFn) || [];
            const basePath = Reflect.getMetadata(index_1.MetadataKeys.BASE_PATH, controller.constructorFn);
            routers.forEach(({ method, path, handlerName, isDefault, isPublic, isAbsolute }) => {
                if (!isDefault && !isAbsolute) {
                    path = basePath + path;
                }
                const paramDoc = isDefault
                    ? (0, generateParamDefault_1.generateParamDefault)(controller, handlerName)
                    : (0, generateParamCustomize_1.generateParamCustomize)(controller, handlerName);
                if (!content.paths[`${path}`]) {
                    content.paths[`${path}`] = {};
                }
                const jsonApi = {
                    tags: [basePath],
                    description: paramDoc.description || "",
                    parameters: paramDoc.parameters,
                    responses: paramDoc.requestResponse,
                };
                if (method != 'get') {
                    jsonApi.requestBody = paramDoc.requestBody;
                }
                if (!isPublic) {
                    jsonApi.security = [
                        {
                            bearerAuth: []
                        }
                    ];
                }
                content.paths[`${path}`][`${method.toLocaleLowerCase()}`] = jsonApi;
                info.push({
                    method: `${method.toLocaleUpperCase()}`,
                    api: `${path}`,
                    handler: `${controller.constructorFn.name}.${String(handlerName)}`,
                    isDefault: !!isDefault,
                    isPublic: !!isPublic
                });
            });
        });
        info.push({
            method: "GET",
            api: "/api-docs",
            handler: "swagger",
            isDefault: true,
            isPublic: true
        });
        console.table(info);
        const outputFile = "./api-doc.json";
        fs.writeFile(outputFile, JSON.stringify(content), "utf8", (err) => {
            if (err) {
                console.error(err);
                return;
            }
            // file written successfully
            (0, child_process_1.exec)(`npx prettier --write ${outputFile}`);
            console.log("success");
        });
    }
}
exports.ApiDoc = ApiDoc;
