"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Controller = void 0;
const constants_1 = require("../metadata/constants");
const di_1 = require("../di");
const metadata_1 = require("../global/metadata");
const controllers_1 = require("../controllers");
const initRoute = (name) => [
    {
        method: constants_1.MethodRouter.GET,
        path: `${name}`,
        handlerName: "get",
        isDefault: true
    },
    {
        method: constants_1.MethodRouter.POST,
        path: `${name}`,
        handlerName: "create",
        isDefault: true
    },
    {
        method: constants_1.MethodRouter.PUT,
        path: `${name}`,
        handlerName: "update",
        isDefault: true
    },
    {
        method: constants_1.MethodRouter.DELETE,
        path: `${name}`,
        handlerName: "delete",
        isDefault: true
    },
];
function Controller(config) {
    return (constructorFn) => {
        (0, di_1.getServiceProvider)().register(constructorFn);
        const name = (config === null || config === void 0 ? void 0 : config.basePath) || "/" + constructorFn.name.toLowerCase().split("controller")[0] + "s";
        Reflect.defineMetadata(constants_1.MetadataKeys.BASE_PATH, (config === null || config === void 0 ? void 0 : config.basePath) || name, constructorFn);
        const controllerClass = constructorFn;
        let routers = Reflect.hasMetadata(constants_1.MetadataKeys.ROUTERS, controllerClass)
            ? Reflect.getMetadata(constants_1.MetadataKeys.ROUTERS, controllerClass)
            : [];
        if (constructorFn.prototype instanceof controllers_1.CRUDController) {
            routers = routers.concat(initRoute(name).map(routerInit => {
                var _a;
                return Object.assign(Object.assign({}, routerInit), { isPublic: !!((_a = config === null || config === void 0 ? void 0 : config.apiPublics) === null || _a === void 0 ? void 0 : _a.includes(routerInit.method)) });
            }).filter((routerInit) => !routers.find((route) => route.path == routerInit.path && route.method == routerInit.method)));
        }
        Reflect.defineMetadata(constants_1.MetadataKeys.ROUTERS, routers, controllerClass);
        (0, metadata_1.getMetadataController)().push({
            constructorFn,
            instance: (0, di_1.getServiceProvider)().getInfo(constructorFn).singletonInstance,
            name,
        });
    };
}
exports.Controller = Controller;
