"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertToPlural = exports.matchUrl = exports.getObjectFromUrl = exports.cloneObj = void 0;
const cloneObj = (obj) => {
    return JSON.parse(JSON.stringify(obj));
};
exports.cloneObj = cloneObj;
const convertUrlToRegex = (url) => {
    const pattern = url.replace(/:[^/]+/g, "(\\w+)").replace(/\?.*/, "");
    const regex = new RegExp('^' + pattern + "$");
    return regex;
};
const getKeyFromUrl = (url) => {
    const regex = /:\w+/g;
    const match = url.match(regex);
    if (match) {
        return match.map(param => param.slice(1));
    }
    return [];
};
const getObjectFromUrl = (urlRegex, urlTarget) => {
    urlTarget = urlTarget.replace(/\?.*/, "");
    const urlKeyRegex = convertUrlToRegex(urlRegex);
    if (urlKeyRegex.test(urlTarget)) {
        const key = getKeyFromUrl(urlRegex);
        const values = urlTarget.match(urlKeyRegex);
        return key.reduce((target, item, index) => {
            target[item] = values[index + 1];
            return target;
        }, {});
    }
    return {};
};
exports.getObjectFromUrl = getObjectFromUrl;
const matchUrl = (urlRegex, urlTarget) => {
    urlTarget = urlTarget.replace(/\?.*/, "");
    const urlKeyRegex = convertUrlToRegex(urlRegex);
    return urlKeyRegex.test(urlTarget);
};
exports.matchUrl = matchUrl;
const convertToPlural = (word) => {
    const irregulars = {
        // irregular words
        child: 'children',
        man: 'men',
        woman: 'women',
        tooth: 'teeth',
        foot: 'feet',
        goose: 'geese',
        mouse: 'mice'
    };
    const rules = [
        // regular rules
        [/(ch|sh|s|x|z)$/i, '$1es'],
        [/[^aeiou]y$/i, 'ies'],
        [/$/, 's']
    ];
    // check irregular words
    if (irregulars[word]) {
        return irregulars[word];
    }
    // check regular rules
    for (const [pattern, replacement] of rules) {
        if (typeof pattern === 'object' && pattern.test(word)) {
            return word.replace(pattern, replacement);
        }
    }
    return word;
};
exports.convertToPlural = convertToPlural;
